import pytest
from unittest.mock import patch
from src.funnygpt.functions import onewordperline, changepreset, gptchat, cowtalk, preset

# A mock response that mimics the OpenAI API response structure
mock_openai_response = {
    'choices': [
        {'message': {'content': 'mocked response'}}
    ]
}


# Mock the llm function
@pytest.fixture
def mock_llm(monkeypatch):
    def mock(*args, **kwargs):
        return 'mocked response'
    monkeypatch.setattr("src.funnygpt.functions.llm", mock)

# Tests for gptchat
@pytest.mark.parametrize("test_input, expected", [
    (("joke", "computers"), "mocked response"),
    (("joke", None), "mocked response"),
    (("haiku", "spring"), "mocked response"),
    (("haiku", None), "mocked response"),
    (("compliment", "music"), "mocked response"),
    (("compliment", None), "mocked response"),
    (("email", "project"), "mocked response"),
    (("email", None), "mocked response"),
    ((None, "a fact"), "mocked response"),
    ((None, None), "mocked response"),
])
def test_gptchat(mock_llm, test_input, expected):
    type_, subject_ = test_input
    response = gptchat(type=type_, subject=subject_)
    assert expected == response

@pytest.mark.parametrize("test_input, expected", [
    (("unknown_type", "anything"), "mocked response"),
    (("unknown_type", None), "mocked response"),
    (("", ""), "mocked response"),
    (("joke", ""), "mocked response"),
    (("", "space"), "mocked response"),
])
def test_gptchat_additional_cases(mock_llm, test_input, expected):
    type_, subject_ = test_input
    response = gptchat(type=type_, subject=subject_)
    assert expected == response


#cowtalk test
# Mock `gptchat` response as it's not directly called from `cowtalk`
@pytest.fixture
def mock_gptchat(monkeypatch):
    def mock(*args, **kwargs):
        # Return the input if provided, else a default string
        return args[0] if args else "This is a mocked response from gptchat function."
    monkeypatch.setattr("src.funnygpt.functions", mock)

# Helper function to create the expected 'moo' pattern
def create_moo_pattern(sentence):
    words = sentence.split()
    moo_pattern = []
    for i in range(0, len(words), 3):
        moo_pattern.extend(words[i:i+3])
        moo_pattern.append("moo")
    return " ".join(moo_pattern).rstrip(' moo')

# Test for a regular sentence with multiples of three words
def test_cowtalk_regular_sentence(mock_gptchat):
    test_input = "This is a test sentence for the cowtalk function."
    expected_output = create_moo_pattern(test_input)
    result = cowtalk(test_input)
    assert result == expected_output

# Test for a sentence with a non-multiple of three words
def test_cowtalk_non_multiple_of_three(mock_gptchat):
    test_input = "One two three four five"
    expected_output = create_moo_pattern(test_input)
    result = cowtalk(test_input)
    assert result == expected_output

# Test for an empty string
def test_cowtalk_empty_string(mock_gptchat):
    test_input = ""
    expected_output = ""  # Assuming no "moo" added if there are no words
    result = cowtalk(test_input)
    assert result == expected_output



# test one wrod per line
# Test cases
def test_onewordperline_normal():
    input_string = "hello world"
    expected_output = "hello\nworld"
    assert onewordperline(input_string) == expected_output

def test_onewordperline_with_punctuation():
    input_string = "hello, world!"
    expected_output = "hello,\nworld!"
    assert onewordperline(input_string) == expected_output

def test_onewordperline_with_multiple_spaces():
    input_string = "hello   world"  # Multiple spaces between words
    expected_output = "hello\nworld"
    assert onewordperline(input_string) == expected_output


def test_changepreset_updates_global_preset(monkeypatch):
    monkeypatch.setattr('builtins.input', lambda _: "New Preset")
    result = changepreset()
    assert result == "New Preset"


def test_changepreset_empty_input(monkeypatch):
    global preset
    # Assume preset has a predefined value before the test
    original_preset = preset
    monkeypatch.setattr('builtins.input', lambda _: "")
    result = changepreset()
    assert result == original_preset  # Expecting the original preset after empty input

def test_changepreset_space_input(monkeypatch):
    global preset
    # Assume preset has a predefined value before the test
    original_preset = preset
    monkeypatch.setattr('builtins.input', lambda _: " ")
    result = changepreset()
    assert result == original_preset  # Expecting the original preset after space input




from openai import OpenAI
client = OpenAI()

def llm(question):
    completion = client.chat.completions.create(
        model="gpt-3.5-turbo",
        messages=[
            {"role": "user", "content": question},
        ],
        max_tokens=200,
        temperature=0.99
    )
    answer = completion.choices[0].message.content

    return answer



preset = "You are the most chill cs professor in the world, Professor Foo Barstein. You are talking to me, a student in your class."


def gptchat(type=None, subject=None, preset=preset):
    if (type == "joke"):
        if (subject):
            response = llm(preset + "tell me a funny joke about " + subject)
        else:
            response = llm(preset + "tell me a funny joke")
    elif (type == "haiku"):
        if (subject):
            response = llm(preset + "tell me a haiku about " + subject)
        else:
            response = llm(preset + "tell me a haiku")
    elif (type == "compliment"):
        if (subject):
            response = llm(preset + "tell me a compliment about " + subject)
        else:
            response = llm(preset + "tell me a compliment")
    elif (type == "email"):
        if (subject):
            response = llm(preset + "write an email about " + subject)
        else:
            response = llm(preset + "write a random email to send to my boss")
    else:
        if (subject):
            response = llm(preset + subject)
        else:
            response = llm(preset + "say something random")
    return response


def cowtalk(input):
    words = input.split()
    # Create a new list that includes 'moo' after every three words
    moo_words = []
    for i in range(0, len(words), 3):
        moo_words.extend(words[i:i+3])  # Add the next three words
        moo_words.append('moo')         # Follow them with 'moo'
    
    # Convert the list back into a string and remove the last 'moo' if it is extra
    moo_sentence = ' '.join(moo_words).rstrip(' moo')
    return moo_sentence


def onewordperline(input_string):
    words = input_string.split()  # Split the input string into words
    return '\n'.join(words)      # Join the words with a newline character


def changepreset():
    global preset  # Use the 'preset' variable defined outside the function
    original_preset=preset
    new_preset = input("Enter the new preset: ")
    if new_preset == "":
        return original_preset
    elif new_preset == " ":
        return original_preset
    else:
        return new_preset



def main():
    # result = onewordperline()
    # Do something with the result if needed, for example:
    # print(result)
    pass


if __name__ == "__main__":
    main()
